<?php

/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Cron;

use FiloBlu\MappConnector\Helper\ExceptionHelper;

/**
 * Class Cron
 *
 * @package FiloBlu\MappConnector\Cron
 */
class Cron {

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\Prepare
     */
    protected $_prepare;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\FtpProcess
     */
    protected $_ftpProcess;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\RestProcess
     */
    protected $_restProcess;

    /**
     * XML PATHS
     */
    const MAPPCONNECTOR_CRON_SUBSCRIBER = "mappconnector/cron/subscriber_enabled";
    const MAPPCONNECTOR_CRON_CUSTOMER = "mappconnector/cron/customer_enabled";
    const MAPPCONNECTOR_CRON_PRODUCT = "mappconnector/cron/product_enabled";
    const MAPPCONNECTOR_CRON_ORDER = "mappconnector/cron/order_enabled";
    const MAPPCONNECTOR_CRON_QUOTE = "mappconnector/cron/quote_enabled";
    const MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE = "mappconnector/cron/subscriber_export_type";
    const MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE = "mappconnector/cron/customer_export_type";

    /**
     * Cron constructor.
     *
     * @param \FiloBlu\MappConnector\Model\Config            $config
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Config $config,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger,
        \FiloBlu\MappConnector\Console\Command\Prepare $prepare,
        \FiloBlu\MappConnector\Console\Command\FtpProcess $ftpProcess,
        \FiloBlu\MappConnector\Console\Command\RestProcess $restProcess
    )
    {
        $this->_fileLogger = $fileLogger;
        $this->_config = $config;
        $this->_prepare = $prepare;
        $this->_ftpProcess = $ftpProcess;
        $this->_restProcess = $restProcess;
    }

    /*
     * PREPARE JOBS SUBSCRIBER
     */
    public function prepareSubscriberIncremental(){
        $this->prepareSubscriberIncrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }
    public function prepareSubscriberFullincremental(){
        $this->prepareSubscriberFullincrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }

    public function prepareSubscriberIncrementalJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1){
            $this->_fileLogger->info('Preparing incremental subscriber '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('subscriber','incremental',$type);
        }
        else {
            $this->_fileLogger->info('Cron incremental subscriber '.$type.' disabled by config');
        }
    }
    public function prepareSubscriberFullincrementalJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental subscriber '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('subscriber', 'fullincremental', $type);
        }
        else {
            $this->_fileLogger->info('Cron fullincremental subscriber '.$type.' disabled by config');
        }
    }
    public function prepareSubscriberFullJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Preparing full subscriber '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('subscriber', 'full', $type);
        }
        else {
            $this->_fileLogger->info('Cron full subscriber '.$type.' disabled by config');
        }
    }

    /*
     * PREPARE JOBS CUSTOMER
     */
    public function prepareCustomerIncremental(){
        $this->prepareCustomerIncrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }
    public function prepareCustomerFullincremental(){
        $this->prepareCustomerFullincrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }

    public function prepareCustomerIncrementalJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing incremental customer '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('customer','incremental',$type);
        }
        else {
            $this->_fileLogger->info('Cron incremental customer '.$type.' disabled by config');
        }
    }
    public function prepareCustomerFullincrementalJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental customer '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('customer','fullincremental',$type);
        }
        else {
            $this->_fileLogger->info('Cron fullincremental customer '.$type.' disabled by config');
        }
    }
    public function prepareCustomerFullJobs($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing full customer '.$type.' from Cron Job');
            $this->_prepare->prepareJobs('customer','full',$type);
        }
        else {
            $this->_fileLogger->info('Cron full customer '.$type.' disabled by config');
        }
    }

    /*
     * PREPARE JOBS QUOTE
     */
    public function prepareQuoteIncremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing incremental quote ftp from Cron Job');
            $this->_prepare->prepareJobs('quote', 'incremental', 'ftp');
        }
        else {
            $this->_fileLogger->info('Cron incremental quote ftp disabled by config');
        }
    }
    public function prepareQuoteFullincremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing fullincremental quote ftp from Cron Job');
            $this->_prepare->prepareJobs('quote','fullincremental','ftp');
        }
        else {
            $this->_fileLogger->info('Cron fullincremental quote ftp disabled by config');
        }
    }
    public function prepareQuoteFull(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing full quote ftp from Cron Job');
            $this->_prepare->prepareJobs('quote','full','ftp');
        }
        else {
            $this->_fileLogger->info('Cron full quote ftp disabled by config');
        }
    }

    /*
     * PREPARE JOBS PRODUCT
     */
    public function prepareProductIncremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing incremental product ftp from Cron Job');
            $this->_prepare->prepareJobs('product','incremental','ftp');
        }
        else {
            $this->_fileLogger->info('Cron incremental product ftp disabled by config');
        }
    }
    public function prepareProductFullincremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing fullincremental product ftp from Cron Job');
            $this->_prepare->prepareJobs('product','fullincremental','ftp');
        }
        else {
            $this->_fileLogger->info('Cron fullincremental product ftp disabled by config');
        }
    }
    public function prepareProductFull(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing full product ftp from Cron Job');
            $this->_prepare->prepareJobs('product','full','ftp');
        }
        else {
            $this->_fileLogger->info('Cron full product ftp disabled by config');
        }
    }

    /*
     * PREPARE JOBS ORDER
     */
    public function prepareOrderIncremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing incremental order ftp from Cron Job');
            $this->_prepare->prepareJobs('order','incremental','ftp');
        }
        else {
            $this->_fileLogger->info('Cron incremental order ftp disabled by config');
        }
    }
    public function prepareOrderFullincremental(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental order ftp from Cron Job');
            $this->_prepare->prepareJobs('order','fullincremental','ftp');
        }
        else {
            $this->_fileLogger->info('Cron fullincremental order ftp disabled by config');
        }
    }
    public function prepareOrderFull(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing full order ftp from Cron Job');
            $this->_prepare->prepareJobs('order','full','ftp');
        }
        else {
            $this->_fileLogger->info('Cron full order ftp disabled by config');
        }
    }

    /*
     * PROCESS FTP JOBS
     */
    public function processSubscriberJob(){
        $this->processSubscriberJobType($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }
    public function processSubscriberJobType($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Process waiting subscriber to '.$type.' ftp from Cron Job');
            if($type == 'ftp')
                $this->_ftpProcess->processFtpJobs('subscriber');
            else if($type == 'rest'){
                $this->_restProcess->processRestJobs('subscriber');
            }
        }
        else {
            $this->_fileLogger->info('Process waiting subscriber to '.$type.' ftp disabled by config');
        }
    }
    public function processCustomerJob(){
        $this->processCustomerJobType($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }
    public function processCustomerJobType($type = 'ftp'){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Process waiting customer to csv '.$type.' from Cron Job');
            if($type == 'ftp')
                $this->_ftpProcess->processFtpJobs('customer');
            else if($type == 'rest'){
                $this->_restProcess->processRestJobs('customer');
            }
        }
        else {
            $this->_fileLogger->info('Process waiting customer to csv '.$type.' disabled by config');
        }
    }
    public function processQuoteJob(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Process waiting quote to csv ftp from Cron Job');
            $this->_ftpProcess->processFtpJobs('quote');
        }
        else {
            $this->_fileLogger->info('Process waiting quote to csv ftp disabled by config');
        }
    }
    public function processProductJob(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Process waiting product to csv ftp from Cron Job');
            $this->_ftpProcess->processFtpJobs('product');
        }
        else {
            $this->_fileLogger->info('Process waiting product to csv ftp disabled by config');
        }
    }
    public function processOrderJob(){
        if($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Process waiting order to csv ftp from Cron Job');
            $this->_ftpProcess->processFtpJobs('order');
        }
        else {
            $this->_fileLogger->info('Process waiting order to csv ftp disabled by config');
        }
    }

}