<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\To\Mapp\Resource;

use \Magento\Framework\DataObject as DataObject;
use FiloBlu\MappConnector\Helper\ObjectHelper;

/**
 * Class UserResource
 * @package FiloBlu\MappConnector\Model\To\Mapp\Resource
 */
class UserResource extends AbstractResource{

    /**
     * @var string
     */
    protected $_defaultadapter = 'FiloBlu\MappConnector\Adapter\RestAdapter';

    /**
     * @param $request
     * @return mixed
     * @throws \Exception
     */
    public function doRequest($request)
    {
        //Set the adapter
        //$this->_defaultadapter = $request->config->adapter;

        //If the email domain is excluded return null
        if($this->_domainIsExcluded($request->request['get']['email'])){
            return null;
        }
       
        //Switch request
        switch ($request->config->type){
            case 'createOrUpdate':
                $response = $this->get($request->request['get'], 'getBy'.ucfirst($request->config->mapp_primary['primaryMapp']));

                $user = json_decode($response->getResponse()->getBody());
                $return = '';
                if(isset($user->id)){
                    //If exists -> update
                    $return = $this->save($request->request, 'updateProfileBy'.ucfirst($request->config->mapp_primary['primaryMapp']));
                }
                else {
                    //If not exists -> create
                    $return = $this->save($request->request, 'create');
                }
                // subscribe/unsubscribe to group id
                if($this->_config->getConfigValue('mappconnector/connection/use_group_id') == 1 && !empty($this->_config->getConfigValue('mappconnector/connection/group_id'))){
                    if(isset($request->request['post']['user.CustomAttribute.IsNewsletterSubscriber']) && $request->request['post']['user.CustomAttribute.IsNewsletterSubscriber'] == 1){
                        $params = [
                            'email' => $request->request['get']['email'],
                            'groupId' => $this->_config->getConfigValue('mappconnector/connection/group_id'),
                            'subscriptionMode' => 'OPT_IN'
                        ];
                        $this->membership($params,'subscribeByEmail');
                    }

                    if(isset($request->request['post']['user.CustomAttribute.IsNewsletterSubscriber']) && $request->request['post']['user.CustomAttribute.IsNewsletterSubscriber'] == 0) {
                        $params = [
                            'email'   => $request->request['get']['email'],
                            'groupId' => $this->_config->getConfigValue('mappconnector/connection/group_id')
                        ];
                        $this->membership($params, 'unsubscribeByEmail');
                    }
                }

                return $return;
                break;
            case 'create':
                return $this->save($request->request, 'create');
                break;
            case 'update':
                return $this->save($request->request, 'updateProfileBy'.ucfirst($request->config->mapp_primary['primaryMapp']));
                break;
            case 'get':
                return $this->get($request->request['get'], 'getBy'.ucfirst($request->config->mapp_primary['primaryMapp']));
                break;
            case 'getProfile':
                return $this->get($request->request['get'], 'getProfileBy'.ucfirst($request->config->mapp_primary['primaryMapp']));
                break;
            case 'delete':
                return $this->delete($request->request, 'deleteBy'.ucfirst($request->config->mapp_primary['primaryMapp']));
                break;
        }
    }

    /**
     * @param $params
     * @param string $method ['get', 'getByEmail', 'getByIdentifier', 'getByMobileNumber']
     * @return mixed
     * @throws \Exception
     */
    public function get($params, $method = 'get'){

        $methods = array('get', 'getByEmail', 'getByIdentifier', 'getByMobileNumber', 'getProfile', 'getProfileByEmail', 'getProfileByMobileNumber');

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method '.$method.' for user/get resourse doesn\'t exists',[],__METHOD__.':'.__LINE__);
        }

        $this->_data->setMethod('GET');
        $this->_data->setEndpoint('/user/'.$method);

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param $params
     * @param string $method ['subscribeByEmail', 'unsubscribeByEmail']
     * @return mixed
     * @throws \Exception
     */
    public function membership($params, $method = 'post'){

        $methods = array('subscribeByEmail', 'unsubscribeByEmail');

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method '.$method.' for user/get resourse doesn\'t exists',[],__METHOD__.':'.__LINE__);
        }

        $this->_data->setMethod('POST');
        $this->_data->setEndpoint('/membership/'.$method);

        $p['get'] = $params;

        parent::makeRequest($p);

        return $this->_data;
    }

    /**
     * @param array $params
     * @param string $method
     * @return mixed
     */
    public function save($params, $method = 'create', $jsonPrepare = true){

        $methods = array('create','updateProfile','updateProfileByEmail','updateProfileByMobileNumber');

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method '.$method.' for user/save resourse doesn\'t exists',[],__METHOD__.':'.__LINE__);
        }

        $this->_data->setMethod('POST');
        $this->_data->setEndpoint('/user/'.$method);

        if($jsonPrepare)
        {
            $params['post'] = json_encode(ObjectHelper::prepareNameValueObjectArray($params['post']));
        }

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param array $params
     * @param string $method
     * @return mixed
     */
    public function delete($params, $method = 'delete'){

        $methods = array('delete','deleteByEmail');

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method '.$method.' for user/get delete doesn\'t exists',[],__METHOD__.':'.__LINE__);
        }

        $this->_data->setMethod('DELETE');
        $this->_data->setEndpoint('/user/'.$method);

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param $id
     * @return mixed
     * @throws \Exception
     */
    public function getUserById($id){
        return $this->get(['userId' => $id]);
    }

    /**
     * @param $email
     * @return mixed
     * @throws \Exception
     */
    public function getUserByEmail($email){
        return $this->get(['email' => $email], "getByEmail");
    }

    /**
     * @param $identifier
     * @return mixed
     * @throws \Exception
     */
    public function getUserByIdentifier($identifier){
        return $this->get(['identifier' => $identifier], "getByIdentifier");
    }

    /**
     * @param $mobileNumber
     * @return mixed
     * @throws \Exception
     */
    public function getUserByMobileNumber($mobileNumber){
        return $this->get(['mobileNumber' => $mobileNumber], "getByMobileNumber");
    }

    /**
     * @param $id
     * @return mixed
     * @throws \Exception
     */
    public function getProfileById($id){
        return $this->get(['userId' => $id], 'getProfile');
    }

    /**
     * @param $email
     * @return mixed
     * @throws \Exception
     */
    public function getProfileByEmail($email){
        return $this->get(['email' => $email], "getProfileByEmail");
    }

    /**
     * @param $mobileNumber
     * @return mixed
     * @throws \Exception
     */
    public function getProfileByMobileNumber($mobileNumber){
        return $this->get(['mobileNumber' => $mobileNumber], "getProfileByMobileNumber");
    }

    /**
     * @param $params
     * @return mixed
     */
    public function create($params){
        return $this->save($params);
    }

    /**
     * @param $params
     * @return mixed
     */
    public function updateProfileById($params){
        $req = ['get' => $params['primary'], 'post' => json_encode(ObjectHelper::prepareNameValueObjectArray($params['info']))];
        return $this->save($req,'updateProfile');
    }

    /**
     * @param $params
     * @return mixed
     */
    public function deleteById($id){
        return $this->delete(['userId' => $id]);
    }

    /**
     * @param $params
     * @return mixed
     */
    public function deleteByEmail($email){
        return $this->delete(['email' => $email], 'deleteByEmail');
    }
}